/*
 * Historical maps for various cities in the Keweenaw.
 * These are more specific than the generic USGS topo maps.
 */

define(["esri/layers/GroupLayer", "esri/layers/TileLayer"], function (GroupLayer, TileLayer) {
    const OLD_MAP_SERVER = "http://gis-core.sabu.mtu.edu:6080/arcgis/rest/services/KeweenawHSDI/";

    const OLD_MAP_NAMES = {
        1880: ["Cal84FIPS", "Htn88_FIPS","Laurium88_FIPS","LL1885FIPS","Quincy1883"],
        1890: ["Cal93FIPS", "Htn93_FIPS","Laurium88_FIPS","LL1893FIPS"],
        1900: ["Cal08FIPS", "Hck07FIPS",  "Htn00_FIPS","Laurium00_FIPS","LL00FIPS"],
        1910: ["Cal17FIPS" ,"Hck17FIPS",  "Htn17_FIPS","Laurium17_FIPS","LL17FIPS"],
        1920: ["Cal28FIPS", "Hck28FIPS",  "Htn17_FIPS", "LL28FIPS"],
        1930: ["Cal28FIPS", "Hck28FIPS",  "Htn28_FIPS","LL35FIPS"],
        1940: ["Mohawk49FIPS", "Cal42FIPS", "Hck42FIPS",  "Htn49_FIPS","Laurium42_FIPS","LL49FIPS",  "Quincy49FIPS", "DollarBay49FIPS", "SoHoCo49_FIPS"],
        1950: ["Mohawk49FIPS", "Cal49FIPS_core","Hancock49FIPS", "Htn49_FIPS","Laurium49FIPS", "LL49FIPS", "DollarBay49FIPS", "Quincy49FIPS", "SoHoCo49_FIPS", "SoRange49_FIPS"]
    };

    /**
     * Create a layer for historical maps around the Keweenaw
     * 
     * @param {*} minScale The minimum scale that the maps are viewable at
     * @param {*} mapView The map view the maps are being displayed on
     */
    return function (minScale, mapView) {
        const layer = new GroupLayer({
            title: "Historical FIPS Maps"
        });

        // Only have maps that are in the current view be visible
        // by checking if the extent intersects with the viewport extent
        mapView.watch("extent", function(ext) {
            layer.layers.forEach(function(layer) {
                if(!ext.intersects(layer.fullExtent)) {
                    layer.visible = false;
                } else {
                    layer.visible = true;
                }
            });
        });

        /**
         * Create a layer for a single map
         * 
         * @param {*} mapName Name of the map to create a layer for
         */
        const createIndividualLayer = function (mapName) {
            return new TileLayer({
                url: OLD_MAP_SERVER + mapName + "/MapServer",
                minScale: minScale
            });
        }

        const self = this;

        /**
         * Set the year represented by the old maps
         * 
         * @param {*} year Year to set the map to. Should be one of 1880, 1890, 1900, 1910, 1920, 1930, 1940, 1950
         */
        layer.setYear = function(year) {
            layer.currentYear = year;

            layer.removeAll();

            const yearMaps = OLD_MAP_NAMES[year];

            // Load all of the maps for the chosen year
            yearMaps.forEach(function(mapTitle) {
                const lyr = createIndividualLayer(mapTitle);

                // Use a promise to handle error when loading map
                lyr.then(function(){}, function(err) {
                    console.error("Error loading old map: " + mapTitle);
                    console.error(err);
                });

                // Add the new map layer to the group
                layer.add(lyr);
            });
        }

        return layer;
    }
});