/*
 * Handles showing a point info when it is selected
 */
define(["esri/layers/GraphicsLayer", "esri/tasks/QueryTask", "esri/tasks/support/Query", "esri/symbols/PictureMarkerSymbol", "esri/Graphic", "./js/PointSubmission.js", "./js/attachments.js"],
    function (GraphicsLayer, QueryTask, Query, PictureMarkerSymbol, Graphic, PointSubmission) {
        // Prefix to put before global IDs for facebook links
        const FACEBOOK_PREFIX = "http://www.csl.mtu.edu/classes/cs4760/www/projects/s17/group1/www/other/GeoDev/html/index.html#";

        const selectedPointSymbol = new PictureMarkerSymbol({
            url: "https://lambda.sx/8FK.svg",
            width: "60px",
            height: "60px"
        });

        const newPointSymbol = new PictureMarkerSymbol({
            url: "http://www.freeiconspng.com/uploads/location-icon-png-14.png",
            width: "48px",
            height: "48px"
        });

        /**
         * @param {*} mapView MapView to handle point selects on
         * @param {*} pointLayer Keweenaw Time Traveller point layer instance to handle
         * @param {*} oldMaps old maps layer
         */
        return function (mapView, pointLayer, oldMaps) {
            const pointSubmission = new PointSubmission(this, pointLayer, oldMaps);

            this.graphicsLayer = new GraphicsLayer();

            let self = this;
            this.selectPoint = function (point) {
                // Center on and zoom to the point
                mapView.goTo({
                    center: [point.graphic.geometry.longitude, point.graphic.geometry.latitude],
                    scale: Math.min(mapView.scale, 16000)
                });

                // Clear the graphics layer from temp points and point overlays
                self.graphicsLayer.removeAll();

                // Add the currently selected point to the graphics layer
                const selectedPointGraphic = $.extend({}, point.graphic);
                selectedPointGraphic.symbol = selectedPointSymbol;
                self.graphicsLayer.add(selectedPointGraphic);

                // Pull off basic attributes from the point
                const graphic = point.graphic;
                const attributes = graphic.attributes;
                const id = attributes["objectid"];
                const gid = attributes["globalid"];

                // Get all of the fields needed to display the point on the sidebar
                const queryTask = new QueryTask({
                    url: pointLayer.url + "/0"
                });
                const query = new Query({
                    where: "objectid=" + id,
                    outFields: ["name", "longtext", "title"] // longtext = description. TODO change
                });
                queryTask.execute(query).then(function (result) {
                    // There should only ever be one result
                    const feature = result.features[0];

                    // Pull off attributes we're interested in
                    const title = feature.attributes["title"];
                    const author = feature.attributes["name"];
                    const descr = feature.attributes["longtext"];

                    // Fill the sidebar with the "show existing point" code
                    $("#sidebar").load("html/existing-point-sidebar.html",
                        function () { // On sidebar content load finish

                            // Show the sidebar
                            $("#sidebar").fadeIn();

                            // Set the title of the sidebar to be the point's title
                            $("#cur-point-name").text(title);

                            // Set the author of the point in the sidebar
                            if (author.length > 0) {
                                $("#cur-point-author").text("By " + author);
                            } else { // If the point is anonymous, hide the author section
                                $("#cur-point-author").hide();
                            }

                            // Set the description of the point in the sidebar
                            if (descr.length > 0) {
                                $("#cur-point-desc").text(descr).linkify(); // Convert links in the description to actual links
                            } else { // If no description, don't show the section
                                $("#cur-point-desc-section").hide();
                            }

                            // Show the image attachments for the point

                            attachmentifyPoint(point); // We need to use attachments, so attachmentify

                            let imageList = $("#cur-point-images");
                            imageList.empty(); // Empty current image list

                            point.getAttachments(function (attachments) {
                                if (attachments == null) {
                                    return;
                                }

                                attachments.forEach(function (attachment) {
                                    // URL that the attachment's data is at
                                    const dataURL = point.attachURL + "/" + attachment["id"];

                                    if (attachment.contentType.startsWith("image/")) { // Only show images
                                        // Create DOM elements for the image
                                        const imageLI = $("<li>");
                                        const imageImg = $("<img>").attr("src", dataURL);

                                        // Enlarge the image when clicked on
                                        imageImg.on("click", function () {
                                            $("#modal-enlarged-image").attr("src", dataURL);
                                            $("#image-modal-label").text(title);
                                            $("#image-modal").modal("show");
                                        });

                                        // Add the image to the DOM
                                        imageLI.append(imageImg);
                                        imageList.append(imageLI);

                                        $("#point-images-section").show();
                                    }
                                });
                            });

                            // make facebook go through the page again
                            // see http://stackoverflow.com/questions/34053622/how-to-change-the-data-href-in-facebook-comments
                            // "The SDK only goes through your document once on initialization – if you change the DOM afterwards, you need to call FB.XFBML.parse to make it go through your document again."
                            const fbComments = document.getElementById("facebook_comments");
                            fbComments.setAttribute("fb-xfbml-state", null); // Fixes the weird bug where half a comment box shows
                            fbComments.setAttribute("data-href", FACEBOOK_PREFIX + gid);
                            FB.XFBML.parse();
                        });
                });
            }

            /**
             * Makes the new point marker and brings up the new point sidebar
             */
            const beginNewPoint = function (screenPoint) {
                self.newPointPos = screenPoint.mapPoint;

                // Deselect currently selected point
                self.graphicsLayer.removeAll();

                // Show indicator for new point
                const newPointMarker = new Graphic({
                    geometry: screenPoint.mapPoint,
                    symbol: newPointSymbol
                });
                self.graphicsLayer.add(newPointMarker);

                // Fill the sidebar with the new point form
                $("#sidebar").load("html/new-point-sidebar.html",
                    function () {
                        $("#sidebar").fadeIn(); // Show the sidebar

                        // Setup image upload fields
                        // TODO cleanup
                        $("#sidebar").find("#imageinput1").change(function () {
                            $("#sidebar").find("#imageinput2").fadeIn();
                        });
                        $("#sidebar").find("#imageinput2").change(function () {
                            $("#sidebar").find("#imageinput3").fadeIn();
                        });
                    });

                // Center on and zoom to the point
                mapView.goTo({
                    center: [self.newPointPos.longitude, self.newPointPos.latitude],
                    scale: Math.min(mapView.scale, 16000)
                });
            }

            const onHitTest = function (response, screenPoint) {
                let foundPoint = false;

                response.results.forEach(function (result) {
                    // Verify that the point is from the correct layer
                    if (result.graphic.layer == pointLayer) {
                        self.selectPoint(result);
                        foundPoint = true;
                    }
                })

                if (!foundPoint) {
                    beginNewPoint(screenPoint);
                }
            }

            const onMouseClick = function (screenPoint) {
                mapView.hitTest(screenPoint).then(function (response) { onHitTest(response, screenPoint) });
            };

            mapView.on("click", onMouseClick);

            // Handle sidebar close
            window.closeSideBar = function () {
                self.graphicsLayer.removeAll();
                $("#sidebar").fadeOut();
            }

            // Close sidebar when escape is pressed
            $(document).bind("keydown", function (e) {
                if (e.keyCode == 27) { // 27 = escape
                    window.closeSideBar();
                }
            })
        }
    });