require([
  "esri/Map",
  "esri/views/MapView",
  "esri/Basemap",
  "esri/geometry/Extent",
  "./js/UsgsHistoricTopoLayer.js",
  "./js/OldMaps.js",
  "./js/DecadeSelectDropdown.js",
  "./js/KTTPointsLayer.js",
  "./js/PointSelectHandler.js",
  "./js/KTTPointSearch.js",
  "dojo/domReady!"
], function (Map, MapView, Basemap, Extent, UsgsHistoricTopoLayer, OldMaps, DecadeSelectDropdown,
  KTTPointsLayer, PointSelectHandler, KTTPointSearch) {

    var usgsTopo = new UsgsHistoricTopoLayer();

    // Create basemap using the USGS topo map
    var basemap = new Basemap({
      baseLayers: [usgsTopo],
      title: "USGS Historical Topographic Maps",
      id: "usgsTopo"
    });

    // Create a map with our basemap
    var map = new Map({
      basemap: basemap
    });

    var mapView = new MapView({
      container: "mapView",  // Reference to the div in index.html
      map: map,
      constraints: {
        minScale: 2000,        // Don't allow users to zoom in too far and get confused
        maxScale: 470000,      // Maps break when zoomed out too far
        snapToZoom: false,     // Allow continuous zoom (uses brower's zoom which will feel better than Esri's)
        rotationEnabled: false // Users get confused by rotation
      },
      extent: new Extent({ // Keweenaw, centered near Houghton w/ zoom
        xmin: -9865527.113397973,
        ymin: 5946031.078471539,
        xmax: -9856606.934066081,
        ymax: 5972217.7248438075,
        spatialReference: 102100
      })
    });

    /*
    The 1945-1955 topo maps are more detailed,
    but the 1985 topo maps are more easily readable at farther zoom.
    Switch between them depending on zoom level.
    */
    mapView.watch("scale", function (newValue) {
      if (newValue <= 62000) {
        usgsTopo.setYearRange(1945, 1955);
      } else {
        usgsTopo.setYearRange(1985, 1985);
      }
    });

    // Setup old maps
    var oldMapsLayer = new OldMaps(25000, mapView);
    var decadeSelectDropdown = new DecadeSelectDropdown(oldMapsLayer);
    map.add(oldMapsLayer);

    // Setup points
    var pointsLayer = new KTTPointsLayer();
    map.add(pointsLayer);

    // Setup point select handler
    var pointSelectHandler = new PointSelectHandler(mapView, pointsLayer, oldMapsLayer);
    map.add(pointSelectHandler.graphicsLayer);

    // Setup search widget
    var pointSearch = new KTTPointSearch(pointsLayer, mapView, pointSelectHandler);
    mapView.ui.add(pointSearch, {
      position: "top-right"
    });

    // Move the zoom icons to the right so they aren't covered by the sidebar
    // This is done after the point search so that it is put below the point search
    mapView.ui.move("zoom", "top-right");
  });